/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandInvalidArgumentException;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.regex.Pattern;

import org.apache.commons.cli.HelpFormatter;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.extension.command.monitoring.exception.SubscriptionObjectNotSubscribedException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringObject;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.DuplicatedMonitoringObjectException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.DuplicatedMonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.InvalidMonitoringObjectNameException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.InvalidMonitoringSubFunctionNameException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.EmittableDefect;
import org.ow2.petals.cli.extension.command.monitoring.so.api.SubscriptionObject;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.DuplicatedEmittableDefectException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.DuplicatedSubscriptionObjectException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.InvalidEmittableDefectNameException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.InvalidSubscriptionObjectNameException;

/**
 * Unit tests of the command 'monitoring'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class MonitoringTest {

    private StringStreamShell shell;

    @Before
    public void createAndConnectShell() {
        this.shell = new StringStreamShell();
        this.shell.onConnectionEstablished();
    }

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {

        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testArgumentsError_0() throws CommandException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        try {
            command.execute(new String[] {});
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testArgumentsError_1() throws CommandException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        try {
            command.execute(new String[] { "-z" });
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the option associated to the monitoring object is set but without
     * value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test(expected = CommandMissingArgumentException.class)
    public void testArgumentsError_2() throws CommandException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        try {
            command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a valid value,</li>
     * <li>the option associated to the sub-function is set but without value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test(expected = CommandMissingArgumentException.class)
    public void testArgumentsError_3() throws CommandException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        try {
            command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION, "dummy",
                    "-" + Monitoring.SUBFONCTION_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to an unknown value,</li>
     * <li>the option associated to the sub-function is set but without value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the unknown value.
     * </p>
     */
    @Test(expected = CommandInvalidArgumentException.class)
    public void testArgumentsError_4() throws CommandException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        final String invalidValue = "unknown-mo";
        try {
            command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION, invalidValue,
                    "-" + Monitoring.SUBFONCTION_SHORT_OPTION, "dummy" });
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, Monitoring.OBJECT_SHORT_OPTION, invalidValue);
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the sub-function is set to an unknown value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the unknown value.
     * </p>
     */
    @Test(expected = CommandInvalidArgumentException.class)
    public void testArgumentsError_5() throws CommandException,
            DuplicatedMonitoringObjectException, IllegalArgumentException,
            InvalidMonitoringObjectNameException, InvalidMonitoringSubFunctionNameException,
            DuplicatedMonitoringSubFunctionException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersMonitoringObjects(new DummyMO());
        final String invalidValue = "unknown-mo";
        try {
            command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION, DummyMO.NAME,
                    "-" + Monitoring.SUBFONCTION_SHORT_OPTION, invalidValue });
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, Monitoring.SUBFONCTION_SHORT_OPTION, invalidValue);
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the sub-function is set to a known value,</li>
     * <li>the subscription object is set for subscription without value.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test(expected = CommandMissingArgumentException.class)
    public void testArgumentsError_6() throws CommandException,
            DuplicatedMonitoringObjectException, IllegalArgumentException,
            InvalidMonitoringObjectNameException, InvalidMonitoringSubFunctionNameException,
            DuplicatedMonitoringSubFunctionException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersMonitoringObjects(new DummyMO());
        try {
            command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION, DummyMO.NAME,
                    "-" + Monitoring.SUBFONCTION_SHORT_OPTION, DummyMO.SUBFCT_NAME,
                    "-" + Monitoring.SUBSCRIPTION_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the sub-function is set to a known value,</li>
     * <li>the subscription object is set, for subscription, to an unknown
     * value.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testArgumentsError_7() throws CommandException,
            DuplicatedMonitoringObjectException, IllegalArgumentException,
            InvalidMonitoringObjectNameException, InvalidMonitoringSubFunctionNameException,
            DuplicatedMonitoringSubFunctionException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersMonitoringObjects(new DummyMO());
        final String invalidValue = "unknown-so";
        try {
            command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION, DummyMO.NAME,
                    "-" + Monitoring.SUBFONCTION_SHORT_OPTION, DummyMO.SUBFCT_NAME,
                    "-" + Monitoring.SUBSCRIPTION_SHORT_OPTION, invalidValue });
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the sub-function is set to a known value,</li>
     * <li>the subscription object is set, for subscription, to an known value.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testArgumentsError_8() throws CommandException,
            DuplicatedMonitoringObjectException, IllegalArgumentException,
            InvalidMonitoringObjectNameException, InvalidMonitoringSubFunctionNameException,
            DuplicatedMonitoringSubFunctionException, DuplicatedSubscriptionObjectException,
            DuplicatedEmittableDefectException, InvalidSubscriptionObjectNameException,
            InvalidEmittableDefectNameException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersMonitoringObjects(new DummyMO());
        command.registersSubscriptionObjects(new DummySO());
        try {
            command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION, DummyMO.NAME,
                    "-" + Monitoring.SUBFONCTION_SHORT_OPTION, DummyMO.SUBFCT_NAME,
                    "-" + Monitoring.SUBSCRIPTION_SHORT_OPTION, DummySO.NAME });
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the option associated to the subscription object is set to subscribe
     * but without value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test(expected = CommandMissingArgumentException.class)
    public void testArgumentsError_9() throws CommandException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        try {
            command.execute(new String[] { "-" + Monitoring.SUBSCRIPTION_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the option associated to the subscription object is set, for
     * subscription, to an unknown value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the the unknown value.
     * </p>
     */
    @Test(expected = CommandInvalidArgumentException.class)
    public void testArgumentsError_10() throws CommandException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        final String invalidValue = "unknown-so";
        try {
            command.execute(new String[] { "-" + Monitoring.SUBSCRIPTION_SHORT_OPTION, invalidValue });
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, Monitoring.SUBSCRIPTION_SHORT_OPTION, invalidValue);
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the sub-function is set to a known value,</li>
     * <li>the subscription object is set for unsubscription without value.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test(expected = CommandMissingArgumentException.class)
    public void testArgumentsError_11() throws CommandException,
            DuplicatedMonitoringObjectException, IllegalArgumentException,
            InvalidMonitoringObjectNameException, InvalidMonitoringSubFunctionNameException,
            DuplicatedMonitoringSubFunctionException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersMonitoringObjects(new DummyMO());
        try {
            command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION, DummyMO.NAME,
                    "-" + Monitoring.SUBFONCTION_SHORT_OPTION, DummyMO.SUBFCT_NAME,
                    "-" + Monitoring.UNSUBSCRIPTION_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the sub-function is set to a known value,</li>
     * <li>the subscription object is set, for unsubscription, to an unknown
     * value.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testArgumentsError_12() throws CommandException,
            DuplicatedMonitoringObjectException, IllegalArgumentException,
            InvalidMonitoringObjectNameException, InvalidMonitoringSubFunctionNameException,
            DuplicatedMonitoringSubFunctionException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersMonitoringObjects(new DummyMO());
        final String invalidValue = "unknown-so";
        try {
            command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION, DummyMO.NAME,
                    "-" + Monitoring.SUBFONCTION_SHORT_OPTION, DummyMO.SUBFCT_NAME,
                    "-" + Monitoring.UNSUBSCRIPTION_SHORT_OPTION, invalidValue });
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the sub-function is set to a known value,</li>
     * <li>the subscription object is set, for unsubscription, to an known
     * value.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testArgumentsError_13() throws CommandException,
            DuplicatedMonitoringObjectException, IllegalArgumentException,
            InvalidMonitoringObjectNameException, InvalidMonitoringSubFunctionNameException,
            DuplicatedMonitoringSubFunctionException, DuplicatedSubscriptionObjectException,
            DuplicatedEmittableDefectException, InvalidSubscriptionObjectNameException,
            InvalidEmittableDefectNameException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersMonitoringObjects(new DummyMO());
        command.registersSubscriptionObjects(new DummySO());
        try {
            command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION, DummyMO.NAME,
                    "-" + Monitoring.SUBFONCTION_SHORT_OPTION, DummyMO.SUBFCT_NAME,
                    "-" + Monitoring.UNSUBSCRIPTION_SHORT_OPTION, DummySO.NAME });
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the option associated to the subscription object is set to
     * unsubscribe but without value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test(expected = CommandMissingArgumentException.class)
    public void testArgumentsError_14() throws CommandException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        try {
            command.execute(new String[] { "-" + Monitoring.UNSUBSCRIPTION_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the option associated to the subscription object is set, for
     * unsubscription, to an unknown value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the the unknown value.
     * </p>
     */
    @Test(expected = CommandInvalidArgumentException.class)
    public void testArgumentsError_15() throws CommandException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        final String invalidValue = "unknown-so";
        try {
            command.execute(new String[] { "-" + Monitoring.UNSUBSCRIPTION_SHORT_OPTION,
                    invalidValue });
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, Monitoring.UNSUBSCRIPTION_SHORT_OPTION, invalidValue);
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the option associated to the subscription object is set, for
     * <b>subscription</b>, to a known value,</li>
     * <li>the option associated to the subscription object is set, for
     * <b>unsubscription</b>, to a known value,</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number argument.
     * </p>
     */
    @Test(expected = CommandInvalidException.class)
    public void testArgumentsError_16() throws CommandException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException,
            IllegalArgumentException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        command.registersSubscriptionObjects(new DummySO());

        try {
            command.execute(new String[] { "-" + Monitoring.SUBSCRIPTION_SHORT_OPTION,
                    DummySO.NAME, "-" + Monitoring.UNSUBSCRIPTION_SHORT_OPTION, DummySO.NAME });
        } catch (final CommandInvalidException e) {
            assertEquals("Unexpected subscription object", command, e.getCommand());
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the option associated to the subscription object is set, for
     * <b>subscription</b>, to a known value,</li>
     * <li>the option associated to the subscription object is set, for
     * <b>unsubscription</b>, to an unknown value,</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number argument.
     * </p>
     */
    @Test(expected = CommandInvalidException.class)
    public void testArgumentsError_17() throws CommandException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException,
            IllegalArgumentException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        command.registersSubscriptionObjects(new DummySO());
        final String invalidValue = "unknown-so";

        try {
            command.execute(new String[] { "-" + Monitoring.SUBSCRIPTION_SHORT_OPTION,
                    DummySO.NAME, "-" + Monitoring.UNSUBSCRIPTION_SHORT_OPTION, invalidValue });
        } catch (final CommandInvalidException e) {
            assertEquals("Unexpected subscription object", command, e.getCommand());
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the option associated to the subscription object is set, for
     * <b>subscription</b>, to a known value,</li>
     * <li>the option associated to the subscription object is set, for
     * <b>unsubscription</b>, without value,</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number argument.
     * </p>
     */
    @Test(expected = CommandInvalidException.class)
    public void testArgumentsError_18() throws CommandException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException,
            IllegalArgumentException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        command.registersSubscriptionObjects(new DummySO());

        try {
            command.execute(new String[] { "-" + Monitoring.SUBSCRIPTION_SHORT_OPTION,
                    DummySO.NAME, "-" + Monitoring.UNSUBSCRIPTION_SHORT_OPTION });
        } catch (final CommandInvalidException e) {
            assertEquals("Unexpected subscription object", command, e.getCommand());
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the option associated to the subscription object is set, for
     * <b>unsubscription</b>, to a known value,</li>
     * <li>the option associated to the subscription object is set, for
     * <b>subscription</b>, to an unknown value,</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number argument.
     * </p>
     */
    @Test(expected = CommandInvalidException.class)
    public void testArgumentsError_19() throws CommandException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException,
            IllegalArgumentException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        command.registersSubscriptionObjects(new DummySO());
        final String invalidValue = "unknown-so";

        try {
            command.execute(new String[] { "-" + Monitoring.UNSUBSCRIPTION_SHORT_OPTION,
                    DummySO.NAME, "-" + Monitoring.SUBSCRIPTION_SHORT_OPTION, invalidValue });
        } catch (final CommandInvalidException e) {
            assertEquals("Unexpected subscription object", command, e.getCommand());
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the option associated to the subscription object is set, for
     * <b>unsubscription</b>, to a known value,</li>
     * <li>the option associated to the subscription object is set, for
     * <b>subscription</b>, without value,</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number argument.
     * </p>
     */
    @Test(expected = CommandInvalidException.class)
    public void testArgumentsError_20() throws CommandException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException,
            IllegalArgumentException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);
        command.registersSubscriptionObjects(new DummySO());

        try {
            command.execute(new String[] { "-" + Monitoring.UNSUBSCRIPTION_SHORT_OPTION,
                    DummySO.NAME, "-" + Monitoring.SUBSCRIPTION_SHORT_OPTION });
        } catch (final CommandInvalidException e) {
            assertEquals("Unexpected subscription object", command, e.getCommand());
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the sub-function is set to a known value,</li>
     * <li>the sub-function needs no arguments.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void testArguments_0() throws CommandException, DuplicatedMonitoringObjectException,
            IllegalArgumentException, InvalidMonitoringObjectNameException,
            InvalidMonitoringSubFunctionNameException, DuplicatedMonitoringSubFunctionException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersMonitoringObjects(new DummyMO());
        command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION, DummyMO.NAME,
                "-" + Monitoring.SUBFONCTION_SHORT_OPTION, DummyMO.SUBFCT_NAME });
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the sub-function is set to a known value,</li>
     * <li>the sub-function needs arguments.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void testArguments_1() throws CommandException, DuplicatedMonitoringObjectException,
            IllegalArgumentException, InvalidMonitoringObjectNameException,
            InvalidMonitoringSubFunctionNameException, DuplicatedMonitoringSubFunctionException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersMonitoringObjects(new DummyMO());
        command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION, DummyMO.NAME,
                "-" + Monitoring.SUBFONCTION_SHORT_OPTION, DummyMO.SUBFCT_NAME, "--", "-z" });
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the subscription object is set, for a subscription, to a known value.
     * </li>
     * </ul>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void testArguments_2() throws CommandException, IllegalArgumentException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersSubscriptionObjects(new DummySO());
        command.execute(new String[] { "-" + Monitoring.SUBSCRIPTION_SHORT_OPTION, DummySO.NAME });
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the subscription object is set, for an unsubscription, to a known
     * value.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void testArguments_3() throws CommandException, IllegalArgumentException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException {
        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersSubscriptionObjects(new DummySO());

        // It is required to do the subscription before unsubscription.
        command.execute(new String[] { "-" + Monitoring.SUBSCRIPTION_SHORT_OPTION, DummySO.NAME });
        command.resetOptions();

        command.execute(new String[] { "-" + Monitoring.UNSUBSCRIPTION_SHORT_OPTION, DummySO.NAME });
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the subscription object is set, for an unsubscription, to a known
     * value,</li>
     * <li>the subscription object has no arguments,
     * <li>the help flag is set,
     * <li>the subscription object has a description,</li>
     * <li>the subscription object has an emittable defect with a description.</li>
     * </ul>
     * <p>
     * Expected results: The help of the subscription object is printed, and
     * contains
     * </p>
     * <ul>
     * <li>the subscription object description,</li>
     * <li>no argument usage,</li>
     * <li>no argument description,</li>
     * <li>the emittable defects with their description.</li>
     * </ul>
     */
    @Test
    public void testArguments_4() throws CommandException, IllegalArgumentException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException {

        // No previous subscription to the subscription object is required to
        // get help
        this.abstractHelpTestOfSO(Monitoring.UNSUBSCRIPTION_SHORT_OPTION,
                new DummySOWithDescription(), false, true);
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the subscription object is set, for a subscription, to a known value,
     * </li>
     * <li>the subscription object has no arguments,
     * <li>the help flag is set,
     * <li>the subscription object has a description,</li>
     * <li>the subscription object has an emittable defect with a description.</li>
     * </ul>
     * <p>
     * Expected results: The help of the subscription object is printed, and
     * contains
     * </p>
     * <ul>
     * <li>the subscription object description,</li>
     * <li>no argument usage,</li>
     * <li>no argument description,</li>
     * <li>the emittable defects with their description.</li>
     * </ul>
     */
    @Test
    public void testArguments_5() throws CommandException, IllegalArgumentException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException {

        this.abstractHelpTestOfSO(Monitoring.SUBSCRIPTION_SHORT_OPTION,
                new DummySOWithDescription(), false, true);
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the subscription object is set, for an unsubscription, to a known
     * value,</li>
     * <li>the subscription object has no arguments,
     * <li>the help flag is set,
     * <li>the subscription object has no description,</li>
     * <li>the subscription object has one emittable defect without description.
     * </li>
     * </ul>
     * <p>
     * Expected results: The help of the subscription object is printed, and
     * contains
     * </p>
     * <ul>
     * <li>no subscription object description,</li>
     * <li>no argument usage,</li>
     * <li>no argument description,</li>
     * <li>the emittable defects with no description.</li>
     * </ul>
     */
    @Test
    public void testArguments_6() throws CommandException, IllegalArgumentException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException {

        // No previous subscription to the subscription object is required to
        // get help
        this.abstractHelpTestOfSO(Monitoring.UNSUBSCRIPTION_SHORT_OPTION, new DummySO(), false,
                false);
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the subscription object is set, for a subscription, to a known value,
     * </li>
     * <li>the subscription object has no arguments,
     * <li>the help flag is set,
     * <li>the subscription object has no description,</li>
     * <li>the subscription object has one emittable defect without description.
     * </li>
     * </ul>
     * <p>
     * Expected results: The help of the subscription object is printed, and
     * contains
     * </p>
     * <ul>
     * <li>no subscription object description,</li>
     * <li>no argument usage,</li>
     * <li>no argument description,</li>
     * <li>the emittable defects with no description.</li>
     * </ul>
     */
    @Test
    public void testArguments_7() throws CommandException, IllegalArgumentException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException {

        this.abstractHelpTestOfSO(Monitoring.SUBSCRIPTION_SHORT_OPTION, new DummySO(), false, false);
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the subscription object is set, for an unsubscription, to a known
     * value,</li>
     * <li>the subscription object has arguments,
     * <li>the help flag is set.</li>
     * <li>the subscription object has no description,</li>
     * <li>the subscription object has one emittable defect without description.
     * </li>
     * </ul>
     * <p>
     * Expected results: The help of the subscription object is printed, and
     * contains
     * </p>
     * <ul>
     * <li>no description,</li>
     * <li>the subscription object argument usage,</li>
     * <li>the subscription object argument description,</li>
     * <li>the emittable defects with no description.</li>
     * </ul>
     */
    @Test
    public void testArguments_8() throws CommandException, IllegalArgumentException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException {

        // No previous subscription to the subscription object is required to
        // get help
        this.abstractHelpTestOfSO(Monitoring.UNSUBSCRIPTION_SHORT_OPTION, new DummySOWithArgs(),
                true, false);
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the subscription object is set, for a subscription, to a known value,
     * </li>
     * <li>the subscription object has arguments,
     * <li>the help flag is set.</li>
     * <li>the subscription object has no description,</li>
     * <li>the subscription object has one emittable defect without description.
     * </li>
     * </ul>
     * <p>
     * Expected results: The help of the subscription object is printed, and
     * contains
     * </p>
     * <ul>
     * <li>no subscription object description,</li>
     * <li>the subscription object argument usage,</li>
     * <li>the subscription object argument description,</li>
     * <li>the emittable defects with no description.</li>
     * </ul>
     */
    @Test
    public void testArguments_9() throws CommandException, IllegalArgumentException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException {

        this.abstractHelpTestOfSO(Monitoring.SUBSCRIPTION_SHORT_OPTION, new DummySOWithArgs(),
                true, false);
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the subscription object is set, for an unsubscription, to a known
     * value,</li>
     * <li>the subscription object has arguments,
     * <li>the help flag is set.</li>
     * <li>the subscription object has a description,</li>
     * <li>the subscription object has one emittable defect with a description.</li>
     * </ul>
     * <p>
     * Expected results: The help of the subscription object is printed, and
     * contains
     * </p>
     * <ul>
     * <li>the subscription object description,</li>
     * <li>the subscription object argument usage,</li>
     * <li>the subscription object argument description,</li>
     * <li>the emittable defects with their description.</li>
     * </ul>
     */
    @Test
    public void testArguments_10() throws CommandException, IllegalArgumentException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException {

        // No previous subscription to the subscription object is required to
        // get help
        this.abstractHelpTestOfSO(Monitoring.UNSUBSCRIPTION_SHORT_OPTION,
                new DummySOWithArgsAndDescr(), true, true);
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the subscription object is set, for a subscription, to a known value,
     * </li>
     * <li>the subscription object has arguments,
     * <li>the help flag is set.</li>
     * <li>the subscription object has a description,</li>
     * <li>the subscription object has one emittable defect with a description.</li>
     * </ul>
     * <p>
     * Expected results: The help of the subscription object is printed, and
     * contains
     * </p>
     * <ul>
     * <li>the subscription object description,</li>
     * <li>the subscription object argument usage,</li>
     * <li>the subscription object argument description,</li>
     * <li>the emittable defects with their description.</li>
     * </ul>
     */
    @Test
    public void testArguments_11() throws CommandException, IllegalArgumentException,
            DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException {

        this.abstractHelpTestOfSO(Monitoring.SUBSCRIPTION_SHORT_OPTION,
                new DummySOWithArgsAndDescr(), true, true);
    }

    private void abstractHelpTestOfSO(final String argument, final SubscriptionObject so,
            final boolean hasArgs, final boolean hasDescription)
            throws DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
            InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException,
            IllegalArgumentException, CommandException {

        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersSubscriptionObjects(so);

        command.execute(new String[] { "-" + argument, so.getName(), "-" + Monitoring.HELP_SHORT_OPTION });

        final String error = this.shell.getError();
        final String output = this.shell.getOutput();

        assertFalse("Empty help message", output.length() == 0);
        assertTrue("An error message was displayed", error.length() == 0);

        final StringBuilder helpMsgRegexp = new StringBuilder();
        helpMsgRegexp.append("^\\n*\\sUSAGE:");
        helpMsgRegexp.append("\\n*\\sSUBSCRIPTION OBJECT:\\p{Blank}").append(so.getName());
        if (hasArgs) {
            assertNotNull("The subscription object MUST have arguments", so.getOptions());
            final HelpFormatter usageFormatter = new HelpFormatter();
            usageFormatter.setSyntaxPrefix("");
            final StringWriter soUsageSw = new StringWriter();
            usageFormatter.printUsage(new PrintWriter(soUsageSw), HelpFormatter.DEFAULT_WIDTH, "", so.getOptions());

            helpMsgRegexp.append("\\n*\\s").append(Pattern.quote(soUsageSw.toString()));
        }
        if (hasDescription) {
            assertNotNull("The subscription object MUST have a description", so.getDescription());
            helpMsgRegexp.append("\\n*\\sDESCRIPTION:");
            helpMsgRegexp.append("\\n*\\s").append(so.getDescription());
        }
        if (hasArgs) {
            assertNotNull("The subscription object MUST have arguments", so.getOptions());
            final StringWriter optionsDescriptionSW = new StringWriter();
            final HelpFormatter helpFormatter = new HelpFormatter();
            helpFormatter.printOptions(new PrintWriter(optionsDescriptionSW), HelpFormatter.DEFAULT_WIDTH,
                    so.getOptions(), HelpFormatter.DEFAULT_LEFT_PAD, HelpFormatter.DEFAULT_DESC_PAD);

            helpMsgRegexp.append("\\n*\\sOPTIONS DESCRIPTION:");
            helpMsgRegexp.append("\\n*\\s").append(Pattern.quote(optionsDescriptionSW.toString()));
        }
        assertNotNull("The subscription object MUST have defects", so.getEmittableDefects());
        assertTrue("The subscription object MUST have at least one defect", so.getEmittableDefects().size() > 0);
        helpMsgRegexp.append("\\n*\\sDEFECT:");
        for (final EmittableDefect emittableDefect : so.getEmittableDefects()) {
            if (hasDescription) {
                assertNotNull("The emittable defect MUST have a description", emittableDefect.getDescription());
                helpMsgRegexp.append("\\n*\\s-\\p{Blank}").append(emittableDefect.getName()).append(":\\p{Blank}")
                        .append(emittableDefect.getDescription());
            } else {
                helpMsgRegexp.append("\\n*\\s-\\p{Blank}").append(emittableDefect.getName());
            }
        }
        helpMsgRegexp.append("\\n*$");
        assertTrue("Unexpected help message: " + output, output.matches(helpMsgRegexp.toString()));
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the monitoring sub-function object is set to a known value,</li>
     * <li>the monitoring sub-function object has arguments,</li>
     * <li>the help flag is set.</li>
     * <li>the monitoring sub-function has a description.</li>
     * </ul>
     * <p>
     * Expected results: The help of the monitoring sub-function is printed, and
     * contains
     * </p>
     * <ul>
     * <li>the monitoring sub-function description,</li>
     * <li>the monitoring sub-function argument usage,</li>
     * <li>the monitoring sub-function argument description.</li>
     * </ul>
     */
    @Test
    public void testArguments_12() throws DuplicatedMonitoringObjectException,
            DuplicatedMonitoringSubFunctionException, InvalidMonitoringObjectNameException,
            IllegalArgumentException, InvalidMonitoringSubFunctionNameException, CommandException {

        this.abstractHelpTestOfMO(new DummyMOWithArgsAndDescr(), true, true);
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the monitoring sub-function object is set to a known value,</li>
     * <li>the monitoring sub-function object has arguments,</li>
     * <li>the help flag is set.</li>
     * <li>the monitoring sub-function has no description.</li>
     * </ul>
     * <p>
     * Expected results: The help of the monitoring sub-function is printed, and
     * contains
     * </p>
     * <ul>
     * <li>no monitoring sub-function description,</li>
     * <li>the monitoring sub-function argument usage,</li>
     * <li>the monitoring sub-function argument description.</li>
     * </ul>
     */
    @Test
    public void testArguments_13() throws DuplicatedMonitoringObjectException,
            DuplicatedMonitoringSubFunctionException, InvalidMonitoringObjectNameException,
            IllegalArgumentException, InvalidMonitoringSubFunctionNameException, CommandException {

        this.abstractHelpTestOfMO(new DummyMOWithArgs(), true, false);
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the monitoring sub-function object is set to a known value,</li>
     * <li>the monitoring sub-function object has no argument,</li>
     * <li>the help flag is set.</li>
     * <li>the monitoring sub-function has no description.</li>
     * </ul>
     * <p>
     * Expected results: The help of the monitoring sub-function is printed, and
     * contains
     * </p>
     * <ul>
     * <li>no monitoring sub-function description,</li>
     * <li>no monitoring sub-function argument usage,</li>
     * <li>no monitoring sub-function argument description.</li>
     * </ul>
     */
    @Test
    public void testArguments_14() throws DuplicatedMonitoringObjectException,
            DuplicatedMonitoringSubFunctionException, InvalidMonitoringObjectNameException,
            IllegalArgumentException, InvalidMonitoringSubFunctionNameException, CommandException {

        this.abstractHelpTestOfMO(new DummyMO(), false, false);
    }

    /**
     * <p>
     * Test of the command "monitoring" against argument management, where:
     * </p>
     * <ul>
     * <li>the monitoring object is set to a known value,</li>
     * <li>the monitoring sub-function object is set to a known value,</li>
     * <li>the monitoring sub-function object has no argument,</li>
     * <li>the help flag is set.</li>
     * <li>the monitoring sub-function has a description.</li>
     * </ul>
     * <p>
     * Expected results: The help of the monitoring sub-function is printed, and
     * contains
     * </p>
     * <ul>
     * <li>a monitoring sub-function description,</li>
     * <li>no monitoring sub-function argument usage,</li>
     * <li>no monitoring sub-function argument description.</li>
     * </ul>
     */
    @Test
    public void testArguments_15() throws DuplicatedMonitoringObjectException,
            DuplicatedMonitoringSubFunctionException, InvalidMonitoringObjectNameException,
            IllegalArgumentException, InvalidMonitoringSubFunctionNameException, CommandException {

        this.abstractHelpTestOfMO(new DummyMOWithDescription(), false, true);
    }

    /*
     * The monitoring object provided MUST contain only one sub-function, the one that will be used
     */
    private void abstractHelpTestOfMO(final MonitoringObject mo, final boolean hasArgs,
            final boolean hasDescription)
            throws DuplicatedMonitoringObjectException, DuplicatedMonitoringSubFunctionException,
            InvalidMonitoringObjectNameException, IllegalArgumentException,
            InvalidMonitoringSubFunctionNameException, CommandException {

        assertEquals("The monitoring object MUST contain only one sub-function", 1, mo.getSubFunctions().size());
        final MonitoringSubFunction subfct = mo.getSubFunctions().get(0);

        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        command.registersMonitoringObjects(mo);

        command.execute(new String[] { "-" + Monitoring.OBJECT_SHORT_OPTION, mo.getName(),
                "-" + Monitoring.SUBFONCTION_SHORT_OPTION, subfct.getName(), "-" + Monitoring.HELP_SHORT_OPTION });

        final String error = this.shell.getError();
        final String output = this.shell.getOutput();

        assertFalse("Empty help message", output.length() == 0);
        assertTrue("An error message was displayed", error.length() == 0);

        final StringBuilder helpMsgRegexp = new StringBuilder();
        helpMsgRegexp.append("^\\n*\\sUSAGE:");
        helpMsgRegexp.append("\\n*\\sMONITORING OBJECT:\\p{Blank}").append(mo.getName());
        helpMsgRegexp.append("\\n\\sSUB-FUNCTION:\\p{Blank}").append(subfct.getName());
        if (hasArgs) {
            assertNotNull("The sub-function MUST have arguments", subfct.getOptions());
            final HelpFormatter usageFormatter = new HelpFormatter();
            usageFormatter.setSyntaxPrefix("");
            final StringWriter usageSw = new StringWriter();
            usageFormatter.printUsage(new PrintWriter(usageSw), HelpFormatter.DEFAULT_WIDTH, "", subfct.getOptions());

            helpMsgRegexp.append("\\n*\\s").append(Pattern.quote(usageSw.toString()));
        }
        if (hasDescription) {
            assertNotNull("The sub-function MUST have a description", subfct.getDescription());
            helpMsgRegexp.append("\\n*\\sDESCRIPTION:");
            helpMsgRegexp.append("\\n*\\s").append(subfct.getDescription());
        }
        if (hasArgs) {
            assertNotNull("The sub-function MUST have arguments", subfct.getOptions());
            final StringWriter optionsDescriptionSW = new StringWriter();
            final HelpFormatter helpFormatter = new HelpFormatter();
            helpFormatter.printOptions(new PrintWriter(optionsDescriptionSW), HelpFormatter.DEFAULT_WIDTH,
                    subfct.getOptions(), HelpFormatter.DEFAULT_LEFT_PAD, HelpFormatter.DEFAULT_DESC_PAD);

            helpMsgRegexp.append("\\n*\\sOPTIONS DESCRIPTION:");
            helpMsgRegexp.append("\\n*\\s").append(Pattern.quote(optionsDescriptionSW.toString()));
        }
        helpMsgRegexp.append("\\n*$");
        assertTrue("Unexpected help message: " + output, output.matches(helpMsgRegexp.toString()));
    }

    /**
     * <p>
     * Check the registration of a monitoring object have two different
     * sub-functions
     * </p>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void testRegistersMonitoringObjects_1() throws DuplicatedMonitoringObjectException,
            InvalidMonitoringObjectNameException, IllegalArgumentException,
            InvalidMonitoringSubFunctionNameException, DuplicatedMonitoringSubFunctionException {
        final Monitoring command = new Monitoring();
        final MonitoringObject monitoringObject = new DummyMO2DifferentSubFcts();

        command.registersMonitoringObjects(monitoringObject);

        assertEquals("Unexpected number of registered sub functions", 2, monitoringObject
                .getSubFunctions().size());

    }

    /**
     * <p>
     * Check the registration of a monitoring object having two sub-functions
     * with the same name
     * </p>
     * <p>
     * Expected results: an error
     * {@link DuplicatedMonitoringSubFunctionException} occurs.
     * </p>
     */
    @Test(expected=DuplicatedMonitoringSubFunctionException.class)
    public void testRegistersMonitoringObjects_2() throws DuplicatedMonitoringObjectException,
            InvalidMonitoringObjectNameException, IllegalArgumentException,
            InvalidMonitoringSubFunctionNameException, DuplicatedMonitoringSubFunctionException {
        final Monitoring command = new Monitoring();
        final MonitoringObject monitoringObject = new DummyMO2SameSubFcts();

        command.registersMonitoringObjects(monitoringObject);
        
        fail("The exception " + DuplicatedMonitoringSubFunctionException.class.getName()
                + " sould occur.");

    }

    /**
     * <p>
     * Check that an error occurs when trying to remove the subscription of an
     * object for which no subscription was previously realized.
     * </p>
     * <ul>
     * <li>the subscription object is set, for an unsubscription, to a known
     * value.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the subscription object without
     * subscription.
     * </p>
     */
    @Test(expected = SubscriptionObjectNotSubscribedException.class)
    public void testUnscriptionWithoutPreviousSubscription() throws CommandException,
            IllegalArgumentException, DuplicatedSubscriptionObjectException,
            DuplicatedEmittableDefectException, InvalidSubscriptionObjectNameException,
            InvalidEmittableDefectNameException {

        final Monitoring command = new Monitoring();
        command.setShell(this.shell);

        final SubscriptionObject dummySO = new DummySO();
        command.registersSubscriptionObjects(dummySO);

        try {
            command.execute(new String[] { "-" + Monitoring.UNSUBSCRIPTION_SHORT_OPTION,
                    DummySO.NAME });
        } catch (final SubscriptionObjectNotSubscribedException e) {
            assertEquals("Unexpected subscription object", dummySO.getName(), e.getSubscriptionObjectName());
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "monitoring" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws CommandException {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new Monitoring();
        command.setShell(shell);

        command.execute(new String[] {});
    }

}
